#!/bin/bash
#########################################################################
#                                                                       #
# ESX/ vSphere restore Script, written by Andreas Jutzy                 #
#                                                                       #
# http://www.aj.de                                                      #
#                                                                       #
# USE WITH CARE AND AT YOUR OWN RISK. CHECK INSTRUCTIONS ON WEBSITE     #
#                                                                       #
# Feel free to modify and contribute good ideas!                        #
#                                                                       #
# Contact info@aj.de for questions and to submit new ideas.             #
#########################################################################

#this script reads all backups
#and offers a menu to restore VMs

#debug variable, as nothing is deleted automatically in this script, debug=1 prints additional info
DEBUG=0

#Version of this script
VERSION=2.31

#where are all your Datastores? defaults to /vmfs/volumes
PATH2LUNS="/vmfs/volumes"

#what is the identifier for your backup Datastores? defaults to BACK
#e.g. LUN0_BACK_PRODUCTION
#or
#e.g. LUN7_SQLMACHINES_BACK
#!!!careful: each Datastore containing the BACKLUNID will be used as a backup Datastore!!!
#!!!THIS MEANS GRACE PERIODS APPLY TO THIS Datastore!!!
BACKLUNID="BACK"

#fill array with ALL Datastoress
ALLLUNS="`ls -al $PATH2LUNS|grep "\->"|awk '{print $9,$11}'`"

if [[ -z $ALLLUNS ]]
then
        echo "ERROR: Could not find ANY Datastore. Aborting..."
        exit 1
fi

#declare arrays and functions
declare -a BACKLUNS

#function to get the backup Datastores using the above declared identifier
function get_backluns
{
        loop=0
        while read lunname lunid
        do
                if [[ `echo $lunname|grep "$BACKLUNID"` ]]
                then
                        if [[ $DEBUG == 1 ]]
                        then
                                echo "Backup Datastore $loop $lunname found..."
                        fi
                        #first quick sanity check of Datastore
                        if [[ -d $PATH2LUNS/$lunname ]]
                        then
                                eval "$lunname"=\'"$lunid"\'
                                BACKLUNS[$loop]="$lunname"
                                let loop=$loop+1
                        else
                                echo "ERROR: Found BACKUP Datastore $PATH2LUNS/$lunname is not sane. Aborting..."
                                exit 1
                        fi

                else
                        if [[ $DEBUG == 1 ]]
                        then
                                echo "Production Datastore $lunname found, skipping..."
                        fi
                fi
        done <<< "$ALLLUNS"
        if  [[ -z ${BACKLUNS[*]} ]]
        then
                echo "ERROR: Could not find a BACKUP Datastore. Aborting..."
                exit 1
        else
                echo -e "Found these backup Datastores ($loop): ${BACKLUNS[*]}\n"
        fi

}

#function to verify BACKUPLUNS
#I implemented that to prevent production datactores to be wiped if something
#went wrong during the detection of datastores
#if a backup datastore does not contain the correct directories, abort
#nothing being wiped in the restore script, but better be safe :D
function verify_backluns ()
{
        for i in ${BACKLUNS[*]}
        do
                if [[ ! -d $PATH2LUNS/$i/ONCE ]] || [[ ! -d $PATH2LUNS/$i/DAILY ]] || [[ ! -d $PATH2LUNS/$i/WEEKLY ]] || [[ ! -d $PATH2LUNS/$i/MONTHLY ]]
                then
                        echo "ERROR: BACKUP Datastore $PATH2LUNS/$i is not correctly prepared or NO BACKUP Datastore. Aborting..."
                        exit 1
                else
                        if [[ $DEBUG = 1 ]]
                        then
                                echo "INFO: BACKUP Datatore $PATH2LUNS/$i is correctly prepared."
                        fi
                fi
        done
}





#declare a array to hold the VMs
declare -a VM

#global counter variable for number of backups
counter=0

get_backluns

verify_backluns


#reading backed up VMs and appending
for i in ${BACKLUNS[*]}
do
        TEMPMACHINES=`find $PATH2LUNS/$i/ -type d|cut -d '/' -f 5-6`
        for j in $TEMPMACHINES
        do
                if [[ ! `echo $j|grep "/"` ]]
                then
                        continue
                fi
                MACHINES="$MACHINES $i/$j"
        done

done

echo -e "I have these Backups:\n"

for line in $MACHINES
do
        let counter=counter+1
        echo -e "$counter)\t$line"
        VM[$counter]="$line"

done
echo -e "0)\tCANCEL\n"

echo -e "\nWhich # do you want to restore?\n"
read VM2RESTORE

if [ "$VM2RESTORE" -ne 0 ]
then
        echo -e "Does the original VM to ${VM[$VM2RESTORE]} exist? (y/n)\n"
        read DOESEXIST
else
        echo -e "Aborting...\n"
        exit -1
fi

if [ "$DOESEXIST" = "y" ]
then
        echo -e "WARNING: Do not proceed until VM is POWERED OFF!!\n"
        echo -e "Is the VM Powered off? (y/n)\n"
        read POWEROFF
        if [ "$POWEROFF" = "n" ]
        then
                echo -e "ERROR: VM has to be powered off! Aborting...\n"
                exit -1
        else
                echo -e "Restoring and overwriting /vmfs/volumes/${VM[$VM2RESTORE]}...\n"
                vcbRestore -s $PATH2LUNS/${VM[$VM2RESTORE]} -b overwrite

        fi
else
        echo -e "Restoring $PATH2LUNS/${VM[$VM2RESTORE]}...\n"
        vcbRestore -s $PATH2LUNS/${VM[$VM2RESTORE]} -b overwrite
fi

exit 0

